import React from 'react';
import { CustomTable } from "../common/table/CustomTable";
import { Grid, Tooltip } from "@material-ui/core";
import { useHistory } from 'react-router-dom';
import { SearchBoxFilter } from '../common/SearchBoxFilter/SearchBoxFilter';
import { IOCStatusIcon } from './IOCIcons';

const ownIocsColumns = [
  { id: "status", label: 'Status', width: '8%' },
  { id: "namingName", label: 'IOC name', width: '20%', sortable: true },
  { id: "host", label: 'Host', width: '15%', sortable: false },
  { id: "network", label: 'Network', width: '15%', sortable: false },
  { id: "sourceVersion", label: 'Git reference', width: '42%', sortable: true },
]

const exploreIocsColumns = [
  { id: "status", label: 'Status', width: '8%' },
  { id: "namingName", label: 'IOC name', width: '20%', sortable: true },
  { id: "host", label: 'Host', width: '15%', sortable: false },
  { id: "network", label: 'Network', width: '15%', sortable: false },
  { id: "owner", label: 'Owner', width: '10%', sortable: true },
  { id: "sourceVersion", label: 'Git reference', width: '32%', sortable: true },
]

const hostDetailsColumns = [
  { id: "status", label: 'Status', width: '8%' },
  { id: "name", label: 'Name', width: '30%', sortable: false },
  { id: "sourceVersion", label: 'Git reference', width: '62%', sortable: false },
]

function createGitVersionField(version, shortVersion) {
  return(
    <>
      {version ?
        <Tooltip title={version} arrow enterDelay={400}>
          <label>{shortVersion}</label>
        </Tooltip>
        : "---"
      }
    </>
  )
}

function createTableRowForHostDetails(ioc) {
  const { id, name, namingName, sourceVersion, sourceVersionShort, alert } = ioc;

  return {
    id: id,
    status: 
      <Grid container direction="column" justifyContent="center" alignItems="center">
        <IOCStatusIcon ioc={ioc} />
      </Grid>,
    name: namingName ?? name,
    sourceVersion: createGitVersionField(sourceVersion, sourceVersionShort),
    discrepancy: alert === "WARNING",
    inconsistentState: alert === "ERROR",
  };
}

function createTableRowForOwnIocs(ioc) {
  const { id, activeDeployment, namingName, sourceVersion, sourceVersionShort, alert} = ioc;

  return {
    id: id,
    status: 
    <Grid container direction="column" justifyContent="center" alignItems="center">
      <IOCStatusIcon ioc={ioc} />
    </Grid>,
    namingName: namingName ?? "---",
    sourceVersion: createGitVersionField(sourceVersion, sourceVersionShort),
    host: (activeDeployment?.host.hostName || "---"),
    network: (activeDeployment?.host.network || "---"),
    discrepancy: alert === "WARNING",
    inconsistentState: alert === "ERROR",
    iocNotDeployed: !ioc.activeDeployment
  };
}

function createTableRowForExploreIocs(ioc) {
  const { id, activeDeployment, namingName, owner, sourceVersion, sourceVersionShort, alert} = ioc;

  return {
    id: id,
    status: 
    <Grid container direction="column" justifyContent="center" alignItems="center">
      <IOCStatusIcon ioc={ioc} />
    </Grid>,
    namingName: namingName ?? "---",
    sourceVersion: createGitVersionField(sourceVersion, sourceVersionShort),
    host: (activeDeployment?.host.hostName || "---"),
    network: (activeDeployment?.host.network || "---"),
    owner: (owner || "---"),
    discrepancy: alert === "WARNING",
    inconsistentState: alert === "ERROR",
    iocNotDeployed: !ioc.activeDeployment
  };
}

export function IOCTable({ iocs, rowType="own", totalCount, lazyParams, setLazyParams, columnSort, setColumnSort, rowsPerPage}) {
  const history = useHistory();

  const onRowClicked = (id) => {
    history.push(`/iocs/${id}`);
  };

  return (
    <>
    {
      rowType === "host" ?
      <CustomTable columns={hostDetailsColumns} totalCount={totalCount} rows={iocs.map(ioc => createTableRowForHostDetails(ioc))} handleRowClick={onRowClicked} 
      lazyParams={lazyParams} setLazyParams={setLazyParams} columnSort={columnSort} setColumnSort={setColumnSort} rowsPerPage={rowsPerPage} />
      : (rowType === "own" ?
      <CustomTable columns={ownIocsColumns} totalCount={totalCount} rows={iocs.map(ioc => createTableRowForOwnIocs(ioc))} handleRowClick={onRowClicked} 
      lazyParams={lazyParams} setLazyParams={setLazyParams} columnSort={columnSort} setColumnSort={setColumnSort} rowsPerPage={rowsPerPage} />
      :
      <CustomTable columns={exploreIocsColumns} totalCount={totalCount} rows={iocs.map(ioc => createTableRowForExploreIocs(ioc))} handleRowClick={onRowClicked} 
      lazyParams={lazyParams} setLazyParams={setLazyParams} columnSort={columnSort} setColumnSort={setColumnSort} rowsPerPage={rowsPerPage} />)
    }
    </>
  );
}

export function SearchableIOCTable({ iocs, totalCount, setQuery, rowType, lazyParams, setLazyParams,
  columnSort, setColumnSort, rowsPerPage}) {
  const filter = (search) => {
    setQuery(search.toLowerCase());
    return (ioc) => {
      return true;
    }
  }

  const renderFilteredIOCs = (filteredIOCs) => 
    <IOCTable iocs={filteredIOCs} rowType={rowType} totalCount={totalCount} 
    lazyParams={lazyParams} setLazyParams={setLazyParams} columnSort={columnSort} setColumnSort={setColumnSort} rowsPerPage={rowsPerPage}/>;

  return (
    <SearchBoxFilter items={iocs} filter={filter} render={renderFilteredIOCs} />
  );
}
